package libinput

  1. Overview
  2. Docs

v1.29

  • Add support for older versions of libinput going back to libinput 1.20. This allows the library to be used on Ubuntu 22.04 and Debian 12.
  • Give a clear compile-time error if the C libinput is too old.
  • The libinput-ocaml version number now indicates which version of libinput it is up-to-date with.

v0.1

  • Initial release.