package libinput

  1. Overview
  2. Docs

Module InputSource

OCaml bindings for libinput. This library is used by e.g. Wayland compositors to support keyboards and mice.

Contexts

Sourcemodule Context : sig ... end

Handles for accessing libinput.

Sourcemodule Interface : sig ... end

Callbacks for opening device files.

Sourcemodule Udev : sig ... end

Minimal bindings to libudev to avoid an external dependency for simple uses of libinput.

Devices

Sourcemodule Device : sig ... end

Input devices.

Sourcemodule Device_group : sig ... end

A group of kernel devices making up one physical device.

Sourcemodule Config_status : sig ... end

Status codes returned when applying configuration settings.

Sourcemodule Seat : sig ... end

A collection of input devices (e.g. mouse and keyboard) to be used together by one person.

Events

Sourcemodule Event : sig ... end

Events (e.g. keypresses) reported to the application from libinput.

Sourcemodule Timestamp : sig ... end

Event timestamps, with microsecond precision.

Sourcemodule Keycode : sig ... end

Keyboard key code.

Tablets

Sourcemodule Tablet_tool : sig ... end

A tool being used by a device with the Device.Capability.t.Tablet_tool capability.

Sourcemodule Mode_group : sig ... end

A mode on a tablet pad is a virtual grouping of functionality.