package libinput

  1. Overview
  2. Docs

Module Input.Tablet_toolSource

A tool being used by a device with the Device.Capability.t.Tablet_tool capability.

Tablet events generated by such a device are bound to a specific tool rather than coming from the device directly. Depending on the hardware it is possible to track the same physical tool across multiple Device.t devices. See get_serial for more details.

Sourcetype t
Sourcemodule Type : sig ... end
Sourceval get_type : t -> Type.t
Sourceval get_tool_id : t -> int64 option

get_tool_id t returns the tool ID for a tool object, if any.

This number identifies the specific type of the tool with more precision than the type returned in get_type.

Sourceval has_pressure : t -> bool
Sourceval has_distance : t -> bool
Sourceval has_tilt : t -> bool
Sourceval has_rotation : t -> bool
Sourceval has_slider : t -> bool
Sourceval has_size : t -> bool
Sourceval has_wheel : t -> bool
Sourceval has_button : t -> int -> bool
Sourceval is_unique : t -> bool

is_unique t indicates whether the physical tool can be uniquely identified by libinput.

If a tool can be uniquely identified, keeping a reference to the tool allows tracking the tool across proximity out sequences and across compatible tablets. See get_serial for more details.

Sourceval get_serial : t -> int64 option

get_serial t returns the serial number of the tool, if any.

Sourcemodule Config : sig ... end

Tool configuration.

Sourceval pp : t Fmt.t